# -*- coding: utf-8 -*-

"""
MEDIA:
    Amandala, Belice
USAGE
    $ cd diarioDeChiapas
    // Si se quiere obtener todas las noticias desde las más actuales hasta las más antiguas. //
    $ scrapy crawl noticias --nolog -s filename=noticias.json
    -------------------------------------------------------------------------------------------------
    // Si se quiere obtener todas las noticias desde las más actuales hasta una fecha específica. //
    $ scrapy crawl noticias --nolog -s filename=noticias.json -a year=2018 -a month=3 -a day=5
    -------------------------------------------------------------------------------------------------
    Después será necesario hacer uso del archivo parse_date_files.py para que las noticias contenidas
    en noticias.json sean separadas en archivos por fecha.
"""

import scrapy, re, json
from datetime import datetime, date
from diarioDeChiapas.items import NoticiasItem


TAG_RE = re.compile(r'<[^>]+>')
def remove_tags(text):
    return TAG_RE.sub('', text)

DAT_RE = re.compile(r'\d{4}\/\d{2}\/\d{2}')


class ImportantData(scrapy.Item):
    continue_searching = scrapy.Field()
    last_link          = scrapy.Field()
    section            = scrapy.Field()
    url                = scrapy.Field()


class QuotesSpider(scrapy.Spider):
    name = "noticias"

    def start_requests(self):
        year  = getattr(self, "year", None)
        month = getattr(self, "month", None)
        day   = getattr(self, "day", None)

        if year is not None and month is not None and day is not None:
            self.stopDate = date(int(year), int(month), int(day))
        else:
            self.stopDate = None

        baseURL = "http://www.diariodechiapas.com/landing/"

        section_list = ["editorial", "portada", "metropoli", "region", "la-roja",
                       "deportes", "boga", "ae", "opinion-dia", "trascendio"]
        # section_list = ["editorial"]

        if self.stopDate is None:
            for s in section_list:
                yield scrapy.Request(url=baseURL + s, callback=self.parse)

        else:
            for s in section_list:
                flow_info = ImportantData()
                flow_info['continue_searching'] = False
                
                request = scrapy.Request(url=baseURL + s, callback=self.parse_with_stop_date)
                request.meta['item'] = flow_info
                
                yield request


    def parse(self, response):
        link_list = response.xpath('//section[@class="page__content"]').css('section.post').xpath('./a[@class="post__link"]/@href').extract()
        section   = response.xpath('//section[@class="wrapper"]/h1').extract_first()
        if section is not None : section = remove_tags(section)

        for link in link_list:
            flow_info = ImportantData()
            flow_info['section'] = section

            request = scrapy.Request(url=link, callback=self.parse_item)
            request.meta['item'] = flow_info

            yield request

        next_page = response.css('div.wp-pagenavi').css('a.nextpostslink').css('::attr(href)').extract_first()
        if next_page is not None:
            yield scrapy.Request(url=next_page, callback=self.parse)


    def parse_with_stop_date(self, response):
        flow_info = response.meta['item']

        if not flow_info['continue_searching']:
            link_list = response.xpath('//section[@class="page__content"]').css('section.post').xpath('./a[@class="post__link"]/@href').extract()

            for link in link_list:
                flow_info = ImportantData()
                flow_info['url'] = response.url
                
                if link == link_list[-1] : flow_info['last_link'] = True
                else: flow_info['last_link'] = False
                
                request = scrapy.Request(url=link, callback=self.parse_item_with_stop_date)
                request.meta['item'] = flow_info
                
                yield request

        else:
            next_page = response.css('div.wp-pagenavi').css('a.nextpostslink').css('::attr(href)').extract_first()
            if next_page is not None:
                flow_info['continue_searching'] = False
                
                request = scrapy.Request(url=next_page, callback=self.parse_with_stop_date)
                request.meta['item'] = flow_info
                
                yield request


    def parse_item(self, response):
        flow_info = response.meta['item']
        item = NoticiasItem()
        text = ''

        title = response.xpath('//section[@class="single__content"]/h1').extract_first()
        if title is not None : title = remove_tags(title)

        for p in response.xpath('//section[@class="single__content"]').css('p').extract():
            text += remove_tags(p) + "\n"

        item['date']  = response.xpath('//meta[@property="article:published_time"]/@content').extract_first()
        item['topic'] = flow_info['section']
        item['title'] = title
        item['text']  = text.strip()
        item['url']   = response.url

        yield item


    def parse_item_with_stop_date(self, response):
        d = response.xpath('//meta[@property="article:published_time"]/@content').extract_first()
        dt = datetime.strptime(d, '%Y-%m-%d').date()

        if dt >= self.stopDate:
            flow_info = response.meta['item']
            item = NoticiasItem()
            text = ''

            item['date'] = datetime.strptime(d, '%Y-%m-%d').isoformat("T")
            item['title'] = remove_tags(response.xpath('//div[@class="active"]/h1/a').extract_first())

            try:
                topic = response.css('div.date').css('span.date').css('a::text').extract()[0]
            except:
                topic = None
            item['topic'] = topic

            for p in response.css('div.content').css('p').extract():
                text += remove_tags(p) + "\n"

            item['text'] = text.strip()

            item['url'] = response.url

            yield item

            if flow_info['last_link']:
                flow_info['continue_searching'] = True
                
                request = scrapy.Request(url=flow_info['url'], callback=self.parse_with_stop_date, dont_filter=True)
                request.meta['item'] = flow_info
                
                yield request
